package cn.iocoder.yudao.module.bpm.service.oa.business;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.business.vo.OaBusinessPageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.business.vo.OaBusinessRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.business.vo.OaBusinessSaveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskApproveReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.task.BpmTaskReturnReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.business.OaBusinessDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.business.OaBusinessMapper;
import cn.iocoder.yudao.module.bpm.enums.DictDataConstants;
import cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.bpm.framework.flowable.core.enums.BpmConstants;
import cn.iocoder.yudao.module.bpm.service.task.BpmTaskService;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.apache.commons.lang3.StringUtils;
import org.flowable.engine.TaskService;
import org.flowable.task.api.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_BUSINESS_NOT_EXISTS;

/**
 * 出差流程信息 Service 实现类
 *
 * @author dp
 */
@Service
@Validated
public class OaBusinessServiceImpl implements OaBusinessService {

    /**
     * 对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_business";

    @Resource
    private OaBusinessMapper oaBusinessMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    private BpmTaskService bpmTaskService;

    @Resource
    private TaskService taskService;

    @Resource
    private FileApi fileApi;

    @Resource
    private AdminUserApi adminUserApi;

    @Resource
    private DeptApi deptApi;

    @Override
    @Transactional
    public Long stagingOaBusiness(OaBusinessSaveReqVO stagingReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        AdminUserRespDTO loginUser = adminUserApi.getUser(loginUserId);
        Objects.requireNonNull(loginUser, "登录用户不能为空");
        // 人信息
        AdminUserRespDTO employee = adminUserApi.getUser(stagingReqVO.getEmployeeId());

        OaBusinessDO oaBusiness = BeanUtils.toBean(stagingReqVO, OaBusinessDO.class);
        if (StringUtils.isBlank(oaBusiness.getBusinessId())) {
            // 创建单据uuid
            String uuid = IdUtil.fastSimpleUUID();
            oaBusiness.setBusinessId(uuid);
        }
        if (employee != null) {
            oaBusiness.setEmployeeId(loginUser.getId());
            oaBusiness.setEmployeeName(loginUser.getNickname());
            oaBusiness.setEmployeePhone(loginUser.getMobile());
            oaBusiness.setDeptId(loginUser.getDeptId());
            oaBusiness.setPosition("员工职位");
            oaBusiness.setUserId(loginUser.getId());
        }
        oaBusiness.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_STAGING);
        oaBusiness.setInfoSource("0");
        oaBusiness.setApplyEmployeeId(loginUser.getId());
        oaBusiness.setApplyEmployeeName(loginUser.getNickname());
        // 暂存不保存审批人信息
        oaBusiness.setStartUserSelectAssignees(null);
        // 保存或更新表单信息
        if (oaBusiness.getId() == null) {
            oaBusinessMapper.insert(oaBusiness);
        } else {
            validateOaBusinessExists(oaBusiness.getId());
            oaBusinessMapper.updateById(oaBusiness);
        }
        // 保存业务uuid到附件中
        saveFileList(stagingReqVO.getFileIdList(), oaBusiness.getBusinessId());

        return oaBusiness.getId();
    }

    @Override
    @Transactional
    public Long commitOaBusiness(OaBusinessSaveReqVO commitReqVO) {
        if (CollectionUtil.isEmpty(commitReqVO.getStartUserSelectAssignees())) {
            throw exception(ErrorCodeConstants.TASK_CREATE_FAIL_NO_START_SELECT_ASSIGNEE);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        AdminUserRespDTO loginUser = adminUserApi.getUser(loginUserId);
        if (loginUser == null) {
            throw exception(ErrorCodeConstants.OA_LOGIN_USER_NOT_EXISTS);
        }
        // 人信息
        AdminUserRespDTO employee = adminUserApi.getUser(commitReqVO.getEmployeeId());
        if (employee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }


        OaBusinessDO oaBusiness = BeanUtils.toBean(commitReqVO, OaBusinessDO.class);
        if (StringUtils.isBlank(oaBusiness.getBusinessId())) {
            // 创建单据uuid
            String uuid = IdUtil.fastSimpleUUID();
            oaBusiness.setBusinessId(uuid);
        }
        oaBusiness.setEmployeeId(employee.getId());
        oaBusiness.setEmployeeName(employee.getNickname());
        oaBusiness.setEmployeePhone(employee.getMobile());
        oaBusiness.setDeptId(employee.getDeptId());
        oaBusiness.setPosition("员工职位");
        oaBusiness.setUserId(loginUser.getId());
        oaBusiness.setInfoSource("0");
        oaBusiness.setApplyEmployeeId(loginUser.getId());
        oaBusiness.setApplyEmployeeName(loginUser.getNickname());
        // 保存或更新表单信息
        if (oaBusiness.getId() == null) {
            oaBusinessMapper.insert(oaBusiness);
        } else {
            OaBusinessDO oaBusinessDO = validateOaBusinessExists(oaBusiness.getId());
            if (StrUtil.isNotBlank(oaBusinessDO.getProcInstId())) {
                throw exception(ErrorCodeConstants.PROCESS_INSTANCE_CREATE_FAIL_HAS_PROCESS);
            }
            oaBusinessMapper.updateById(oaBusiness);
        }
        // 发起流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        processInstanceVariables.put("auditPass", "true");
        // 添加审批人信息
        Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
        List<Long> selectAssignees = commitReqVO.getStartUserSelectAssignees();
        startUserSelectAssignees.put("approver", selectAssignees);
        String processInstanceId = processInstanceApi
                .createProcessInstance(loginUser.getId(),
                        new BpmProcessInstanceCreateReqDTO()
                                .setProcessDefinitionKey(PROCESS_KEY)
                                .setVariables(processInstanceVariables)
                                .setBusinessKey(String.valueOf(oaBusiness.getId()))
                                .setStartUserSelectAssignees(startUserSelectAssignees));

        //获取下一个审批人
        Task task = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();

        // 审批同意
        BpmTaskApproveReqVO agreeReqVO = new BpmTaskApproveReqVO();
        agreeReqVO.setId(task.getId());
        agreeReqVO.setReason("[首次提交]");
        agreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_COMMITTED));
        taskService.setVariable(task.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_COMMITTED);
        taskService.setVariable(task.getId(), "auditPass", "true");
        bpmTaskService.approveTask(loginUserId, agreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(processInstanceId).singleResult();
        if (nextTask == null) {
            throw exception(ErrorCodeConstants.TASK_CREATE_FAIL_NO_START_SELECT_ASSIGNEE);
        }
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);


        // 将工作流的流程实例ID、单据状态、最后一次审批时间、当前审批人更新到单据信息中
        oaBusinessMapper.updateById(new OaBusinessDO()
                .setId(oaBusiness.getId())
                .setProcInstId(processInstanceId)
                .setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_COMMITTED)
                .setFinalAuditDate(LocalDateTime.now())
                .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                .setCurrentAuditEmployeeName(currentAuditEmployee.getNickname())
                .setStartUserSelectAssignees(selectAssignees.stream().map(String::valueOf).collect(Collectors.joining(","))));
        // 保存业务uuid到附件中
        saveFileList(commitReqVO.getFileIdList(), oaBusiness.getBusinessId());
        // TODO DP 发送提交成功站内信
        return oaBusiness.getId();
    }

    @Override
    @Transactional
    public Long agreeOaBusiness(BpmTaskApproveReqVO agreeReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        Task currentTask = bpmTaskService.getTask(agreeReqVO.getId());
        if (currentTask == null) {
            throw exception(ErrorCodeConstants.TASK_NOT_EXISTS);
        }
        // 先更新为审批中
        taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITING);
        LambdaQueryWrapper<OaBusinessDO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(OaBusinessDO::getProcInstId, currentTask.getProcessInstanceId());
        OaBusinessDO oaBusiness = oaBusinessMapper.selectOne(lambdaQueryWrapper);
        // 如果是最后一个人审批同意，设置流程审批状态为已审核
        String[] auditPersons = oaBusiness.getStartUserSelectAssignees().split(",");
        String lastAuditPerson = Arrays.stream(auditPersons)
                .reduce((first, second) -> second)
                .orElse(null);
        if (currentTask.getAssignee().equals(lastAuditPerson)) {
            // 更新为已审核(暂时未生效，未找到原因，先不处理)
            taskService.setVariable(agreeReqVO.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITED);
        }
        // 审批同意
        String reason = "[同意]";
        if (StrUtil.isNotBlank(agreeReqVO.getReason())) {
            reason = reason + agreeReqVO.getReason();
        }
        agreeReqVO.setReason(reason);
        agreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_AUDITING));
        taskService.setVariable(agreeReqVO.getId(), "auditPass", "true");
        bpmTaskService.approveTask(loginUserId, agreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        OaBusinessDO oaBusinessDO = new OaBusinessDO();
        if (nextTask != null) {
            Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
            AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);

            // 如果审批人重复，实际未结束，再次更新为审核中
            taskService.setVariable(nextTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_AUDITING);
            oaBusinessDO.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_AUDITING)
                    .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                    .setCurrentAuditEmployeeName(currentAuditEmployee.getNickname())
                    .setFinalAuditDate(LocalDateTime.now())
                    .setId(oaBusiness.getId());

        } else {
            oaBusinessDO.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_AUDITED)
                    .setCurrentAuditEmployeeId(null)
                    .setCurrentAuditEmployeeName(null)
                    .setFinalAuditDate(LocalDateTime.now())
                    .setId(oaBusiness.getId());

            // TODO 复制业务单据信息到业务模块单据表

        }
        oaBusinessMapper.updateById(oaBusinessDO);
        // 发送通知

        // 返回
        return 1L;
    }

    @Override
    @Transactional
    public Long disagreeOaBusiness(BpmTaskReturnReqVO disagreeReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        Task currentTask = bpmTaskService.getTask(disagreeReqVO.getId());

        disagreeReqVO.setTargetTaskDefinitionKey("modifyApply");
        disagreeReqVO.setReason("[驳回]" + disagreeReqVO.getReason());
        disagreeReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_RETURNED));
        taskService.setVariable(disagreeReqVO.getId(), "auditPass", "false");
        taskService.setVariable(disagreeReqVO.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_RETURNED);
        bpmTaskService.returnTask(loginUserId, disagreeReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaBusinessDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaBusinessDO::getCurrentAuditEmployeeId, currentAuditEmployeeId)
                .set(OaBusinessDO::getCurrentAuditEmployeeName, currentAuditEmployee.getNickname())
                .set(OaBusinessDO::getFinalAuditDate, LocalDateTime.now())
                .set(OaBusinessDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_RETURNED)
                .eq(OaBusinessDO::getProcInstId, currentTask.getProcessInstanceId());
        oaBusinessMapper.update(updateWrapper);

        // 发送通知

        return 1L;
    }

    @Override
    @Transactional
    public Long revocationOaBusiness(BpmTaskApproveReqVO revocationReqVO) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        Task currentTask = bpmTaskService.getTask(revocationReqVO.getId());
        if (currentTask == null) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.TASK_NOT_EXISTS);
        }
        LambdaQueryWrapper<OaBusinessDO> lambdaQueryWrapper = new LambdaQueryWrapper<OaBusinessDO>()
                .eq(OaBusinessDO::getProcInstId, currentTask.getProcessInstanceId());
        OaBusinessDO oaBusinessDO = oaBusinessMapper.selectOne(lambdaQueryWrapper);
        if (!DictDataConstants.OA_AUDIT_STATUS_COMMITTED.equals(oaBusinessDO.getAuditStatus())
                || !Objects.equals(String.valueOf(loginUserId), oaBusinessDO.getCreator())) {
            throw ServiceExceptionUtil.exception(ErrorCodeConstants.TASK_REVOCATION_NOT_ALLOWED);
        }

        BpmTaskReturnReqVO returnReqVO = BeanUtils.toBean(revocationReqVO, BpmTaskReturnReqVO.class);
        returnReqVO.setTargetTaskDefinitionKey("modifyApply");
        String reason = "[撤回]";
        if (StrUtil.isNotBlank(returnReqVO.getReason())) {
            reason = reason + returnReqVO.getReason();
        }
        returnReqVO.setReason(reason);
        returnReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_RECALLED));
        taskService.setVariable(currentTask.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_RECALLED);
        bpmTaskService.revocationTask(loginUserId, returnReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);

        // 更新单据状态，当前处理人，最后处理时间
        LambdaUpdateWrapper<OaBusinessDO> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(OaBusinessDO::getCurrentAuditEmployeeId, currentAuditEmployeeId)
                .set(OaBusinessDO::getCurrentAuditEmployeeName, currentAuditEmployee.getNickname())
                .set(OaBusinessDO::getFinalAuditDate, LocalDateTime.now())
                .set(OaBusinessDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_RECALLED)
                .eq(OaBusinessDO::getProcInstId, currentTask.getProcessInstanceId());
        oaBusinessMapper.update(updateWrapper);

        // 发送通知

        return 1L;
    }

    @Override
    @Transactional
    public Long reCommitOaBusiness(OaBusinessSaveReqVO reCommitReqVO) {
        if (reCommitReqVO.getId() == null) {
            throw exception(ErrorCodeConstants.SERVICE_ID_NOT_EXISTS);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        AdminUserRespDTO loginUser = adminUserApi.getUser(loginUserId);
        if (loginUser == null) {
            throw exception(ErrorCodeConstants.OA_LOGIN_USER_NOT_EXISTS);
        }
        // 人信息
        AdminUserRespDTO employee = adminUserApi.getUser(reCommitReqVO.getEmployeeId());
        if (employee == null) {
            throw exception(ErrorCodeConstants.OA_EMPLOYEE_NOT_EXISTS);
        }

        OaBusinessDO oaBusinessNew = BeanUtils.toBean(reCommitReqVO, OaBusinessDO.class);
        OaBusinessDO oaBusinessOld = validateOaBusinessExists(oaBusinessNew.getId());
        oaBusinessNew.setBusinessId(oaBusinessOld.getBusinessId())
                .setProcInstId(oaBusinessOld.getProcInstId());

        oaBusinessNew.setEmployeeId(employee.getId());
        oaBusinessNew.setEmployeeName(employee.getNickname());
        oaBusinessNew.setEmployeePhone(employee.getMobile());
        oaBusinessNew.setDeptId(employee.getDeptId());
        oaBusinessNew.setPosition("员工职位");
        oaBusinessNew.setUserId(loginUser.getId());
        oaBusinessNew.setApplyEmployeeId(loginUser.getId());
        oaBusinessNew.setApplyEmployeeName(loginUser.getNickname());

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaBusinessNew.getProcInstId()).singleResult();
        BpmTaskApproveReqVO approveReqVO = new BpmTaskApproveReqVO();
        approveReqVO.setId(currentTask.getId())
                .setReason("[再次提交]");
        approveReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_COMMITTED));
        if (CollectionUtil.isNotEmpty(reCommitReqVO.getStartUserSelectAssignees())) {
            // 添加审批人信息到流程参数中
            Map<String, List<Long>> startUserSelectAssignees = new HashMap<>();
            List<Long> selectAssignees = reCommitReqVO.getStartUserSelectAssignees();
            startUserSelectAssignees.put("approver", selectAssignees);
            taskService.setVariable(currentTask.getId(), BpmConstants.PROCESS_INSTANCE_VARIABLE_START_USER_SELECT_ASSIGNEES, startUserSelectAssignees);
            // 保存审批人信息
            oaBusinessNew.setStartUserSelectAssignees(selectAssignees.stream().map(String::valueOf).collect(Collectors.joining(",")));
        }
        taskService.setVariable(currentTask.getId(), "auditPass", "true");
        taskService.setVariable(currentTask.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_COMMITTED);
        // 再次提交，和审批通过逻辑相同
        bpmTaskService.approveTask(loginUserId, approveReqVO);

        Task nextTask = taskService.createTaskQuery().processInstanceId(currentTask.getProcessInstanceId()).singleResult();
        Long currentAuditEmployeeId = Long.valueOf(nextTask.getAssignee());
        AdminUserRespDTO currentAuditEmployee = adminUserApi.getUser(currentAuditEmployeeId);
        oaBusinessNew.setAuditStatus(DictDataConstants.OA_AUDIT_STATUS_COMMITTED)
                .setCurrentAuditEmployeeId(currentAuditEmployeeId)
                .setCurrentAuditEmployeeName(currentAuditEmployee.getNickname())
                .setFinalAuditDate(LocalDateTime.now());
        oaBusinessMapper.updateById(oaBusinessNew);

        // 保存业务uuid到附件中
        saveFileList(reCommitReqVO.getFileIdList(), oaBusinessNew.getBusinessId());

        return 1L;
    }

    @Override
    @Transactional
    public Long closeOaBusiness(Long id) {
        if (id == null) {
            throw exception(ErrorCodeConstants.SERVICE_ID_NOT_EXISTS);
        }
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        OaBusinessDO oaBusinessDO = validateOaBusinessExists(id);
        if (!DictDataConstants.OA_AUDIT_STATUS_RETURNED.equals(oaBusinessDO.getAuditStatus())
                && !DictDataConstants.OA_AUDIT_STATUS_RECALLED.equals(oaBusinessDO.getAuditStatus())) {
            throw exception(ErrorCodeConstants.TASK_CLOSE_NOT_ALLOWED);
        }

        Task currentTask = taskService.createTaskQuery().processInstanceId(oaBusinessDO.getProcInstId()).singleResult();
        BpmTaskApproveReqVO approveReqVO = new BpmTaskApproveReqVO();
        approveReqVO.setId(currentTask.getId())
                .setReason("[关闭]");
        approveReqVO.setTaskStatus(Integer.valueOf(DictDataConstants.OA_AUDIT_STATUS_CLOSED));
        taskService.setVariable(currentTask.getId(), "auditPass", "false");
        taskService.setVariable(currentTask.getId(),BpmConstants.PROCESS_INSTANCE_VARIABLE_STATUS, DictDataConstants.OA_AUDIT_STATUS_CLOSED);
        // 关闭，使用审批通过的方法实现
        bpmTaskService.approveTask(loginUserId, approveReqVO);

        LambdaUpdateWrapper<OaBusinessDO> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        lambdaUpdateWrapper.set(OaBusinessDO::getAuditStatus, DictDataConstants.OA_AUDIT_STATUS_CLOSED)
                .set(OaBusinessDO::getCurrentAuditEmployeeId, null)
                .set(OaBusinessDO::getCurrentAuditEmployeeName, null)
                .set(OaBusinessDO::getFinalAuditDate, LocalDateTime.now())
                .eq(OaBusinessDO::getId, id);
        oaBusinessMapper.update(lambdaUpdateWrapper);
        return 1L;
    }

    @Override
    public void deleteOaBusiness(Long id) {
        // 校验存在
        validateOaBusinessDelete(id);
        // 删除
        oaBusinessMapper.deleteById(id);
    }

    @Override
    public OaBusinessRespVO getOaBusiness(Long id) {
        // 登录人信息
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        // TODO DP 根据登录人查询出对应的员工信息

        OaBusinessDO oaBusinessDO = oaBusinessMapper.selectById(id);
        OaBusinessRespVO oaBusinessRespVO = BeanUtils.toBean(oaBusinessDO, OaBusinessRespVO.class);
        if (oaBusinessDO.getDeptId() != null) {
            DeptRespDTO dept = deptApi.getDept(oaBusinessDO.getDeptId());
            if (dept != null) {
                oaBusinessRespVO.setDeptName(dept.getName());
            }
        }

        String procInstId = oaBusinessDO.getProcInstId();
        if (StrUtil.isNotBlank(procInstId)) {
            Task task = taskService.createTaskQuery()
                    .processInstanceId(procInstId)
                    .taskAssignee(String.valueOf(loginUserId))
                    .singleResult();
            if (DictDataConstants.OA_AUDIT_STATUS_COMMITTED.equals(oaBusinessDO.getAuditStatus())) {
                // 如果是已提交，不限制任务处理人是当前登录人，用于直接撤回
                task = taskService.createTaskQuery()
                        .processInstanceId(procInstId)
                        .singleResult();
            }
            if (task != null) {
                oaBusinessRespVO.setTaskId(task.getId());
            }
        }

        // 附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(oaBusinessDO.getBusinessId());
        oaBusinessRespVO.setFileList(fileList);

        // 审批记录
        if (StrUtil.isNotBlank(oaBusinessDO.getProcInstId())) {
            List<BpmTaskRespVO> auditRecordList = bpmTaskService.getAuditRecordListByProcessInstanceId(oaBusinessDO.getProcInstId());
            oaBusinessRespVO.setAuditRecordList(auditRecordList);
        }

        // 审批人
        String startUserSelectAssignees = oaBusinessDO.getStartUserSelectAssignees();
        if (StrUtil.isNotEmpty(startUserSelectAssignees)) {
            List<Long> userIdList = Arrays.stream(startUserSelectAssignees.split(","))
                    .map(Long::valueOf).collect(Collectors.toList());
            List<AdminUserRespDTO> userList = adminUserApi.getUserList(userIdList);
            List<AdminUserRespDTO> auditUserList = new ArrayList<>();
            for (Long userId : userIdList) {
                for (AdminUserRespDTO adminUserRespDTO : userList) {
                    if (Objects.equals(userId, adminUserRespDTO.getId())) {
                        auditUserList.add(adminUserRespDTO);
                        break;
                    }
                }
            }
            oaBusinessRespVO.setAuditUserList(auditUserList);
        }

        return oaBusinessRespVO;
    }

    @Override
    public OaBusinessRespVO getOaBusinessByProcInstId(String procInstId) {
        OaBusinessDO oaBusinessDO = oaBusinessMapper.selectOne(OaBusinessDO::getProcInstId, procInstId);
        OaBusinessRespVO oaBusinessRespVO = BeanUtils.toBean(oaBusinessDO, OaBusinessRespVO.class);
        if (oaBusinessDO.getDeptId() != null) {
            DeptRespDTO dept = deptApi.getDept(oaBusinessDO.getDeptId());
            if (dept != null) {
                oaBusinessRespVO.setDeptName(dept.getName());
            }
        }

        // 附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(oaBusinessDO.getBusinessId());
        oaBusinessRespVO.setFileList(fileList);

        // 审批记录
        if (StrUtil.isNotBlank(oaBusinessDO.getProcInstId())) {
            List<BpmTaskRespVO> auditRecordList = bpmTaskService.getAuditRecordListByProcessInstanceId(oaBusinessDO.getProcInstId());
            oaBusinessRespVO.setAuditRecordList(auditRecordList);
        }

        // 审批人
        String startUserSelectAssignees = oaBusinessDO.getStartUserSelectAssignees();
        if (StrUtil.isNotEmpty(startUserSelectAssignees)) {
            List<Long> userIdList = Arrays.stream(startUserSelectAssignees.split(","))
                    .map(Long::valueOf).collect(Collectors.toList());
            List<AdminUserRespDTO> userList = adminUserApi.getUserList(userIdList);
            List<AdminUserRespDTO> auditUserList = new ArrayList<>();
            for (Long userId : userIdList) {
                for (AdminUserRespDTO adminUserRespDTO : userList) {
                    if (Objects.equals(userId, adminUserRespDTO.getId())) {
                        auditUserList.add(adminUserRespDTO);
                        break;
                    }
                }
            }
            oaBusinessRespVO.setAuditUserList(auditUserList);
        }

        return oaBusinessRespVO;
    }

    @Override
    public PageResult<OaBusinessRespVO> getOaBusinessPage(OaBusinessPageReqVO pageReqVO) {
        PageResult<OaBusinessDO> oaBusinessDOPageResult = oaBusinessMapper.selectPage(pageReqVO);
        PageResult<OaBusinessRespVO> oaBusinessRespVOPageResult = BeanUtils.toBean(oaBusinessDOPageResult, OaBusinessRespVO.class);

        List<OaBusinessRespVO> oaBusinessRespVOList = oaBusinessRespVOPageResult.getList();
        if (CollectionUtil.isNotEmpty(oaBusinessRespVOList)) {
            List<Long> employeeIdList = oaBusinessRespVOList.stream().map(OaBusinessRespVO::getCurrentAuditEmployeeId).collect(Collectors.toList());
            List<AdminUserRespDTO> employeeList = adminUserApi.getUserList(employeeIdList);
            for (OaBusinessRespVO respVO : oaBusinessRespVOList) {
                for (AdminUserRespDTO employee : employeeList) {
                    if (employee.getId() != null && employee.getId().equals(respVO.getCurrentAuditEmployeeId())) {
                        respVO.setCurrentAuditEmployeeName(employee.getNickname());
                        break;
                    }
                }
            }

            List<Long> deptIdList = oaBusinessRespVOList.stream().map(OaBusinessRespVO::getDeptId).collect(Collectors.toList());
            List<DeptRespDTO> deptList = deptApi.getDeptList(deptIdList);
            for (OaBusinessRespVO respVO : oaBusinessRespVOList) {
                for (DeptRespDTO dept : deptList) {
                    if (dept.getId() != null && dept.getId().equals(respVO.getDeptId())) {
                        respVO.setDeptName(dept.getName());
                        break;
                    }
                }
            }
        }

        return oaBusinessRespVOPageResult;
    }

    @Override
    public PageResult<OaBusinessRespVO> exportOaBusinessExcel(OaBusinessPageReqVO pageReqVO) {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        return getOaBusinessPage(pageReqVO);
    }

    private void validateOaBusinessDelete(Long id) {
        OaBusinessDO oaBusinessDO = validateOaBusinessExists(id);
        if (!DictDataConstants.OA_AUDIT_STATUS_STAGING.equals(oaBusinessDO.getAuditStatus())) {
            throw exception(ErrorCodeConstants.DELETE_FAIL_NOT_STAGING);
        }
    }

    private OaBusinessDO validateOaBusinessExists(Long id) {
        OaBusinessDO oaBusinessDO = oaBusinessMapper.selectById(id);
        if (oaBusinessDO == null) {
            throw exception(OA_BUSINESS_NOT_EXISTS);
        }
        return oaBusinessDO;
    }

    /**
     * 保存业务uuid到附件中
     */
    private void saveFileList(List<Long> fileIdList, String serviceId) {
        if (CollectionUtil.isNotEmpty(fileIdList) && StrUtil.isNotEmpty(serviceId)) {
            fileApi.updateFileBiz(fileIdList, serviceId);
        }
    }

}