package cn.iocoder.yudao.module.relations.service.contract;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.tenant.core.aop.TenantIgnore;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import cn.iocoder.yudao.module.relations.controller.admin.contract.vo.RelationsContractPageReqVO;
import cn.iocoder.yudao.module.relations.controller.admin.contract.vo.RelationsContractRespVO;
import cn.iocoder.yudao.module.relations.controller.admin.contract.vo.RelationsContractSaveReqVO;
import cn.iocoder.yudao.module.relations.dal.dataobject.contract.RelationsContractDO;
import cn.iocoder.yudao.module.relations.dal.mysql.contract.RelationsContractMapper;
import cn.iocoder.yudao.module.relations.enums.contractinfo.ContractInfoStatusEnum;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import javax.annotation.Resource;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.relations.enums.ErrorCodeConstants.RELATIONS_CONTRACT_INFO_NOT_EXISTS;

/**
 * 员工合同信息 Service 实现类
 *
 * @author zhaopq
 */
@Service
@Validated
public class RelationsContractServiceImpl implements RelationsContractService {

    @Resource
    private RelationsContractMapper contractMapper;
    @Resource
    private FileApi fileApi;

    @Override
    @TenantIgnore
    public Long createContractInfo(RelationsContractSaveReqVO createReqVO) {
        // 查询该员工是否有合同存在
        List<RelationsContractDO> list = contractMapper.selectPage(new RelationsContractPageReqVO().setEmployeeId(createReqVO.getEmployeeId())).getList();
        // 检查查询结果是否为空
        if (!CollUtil.isEmpty(list)) {
            // 更新状态
            List<RelationsContractDO> toUpdate = new ArrayList<>();
            for (RelationsContractDO contract : list) {
                contract.setStatus(ContractInfoStatusEnum.HISTORY.getStatus());
                toUpdate.add(contract);
            }
            // 批量更新
            if (!toUpdate.isEmpty()) {
                contractMapper.updateBatch(toUpdate);
            }
        }
        // 插入
        String infoId = IdUtil.fastSimpleUUID();
        createReqVO.setContractId(infoId);
        RelationsContractDO contractInfo = BeanUtils.toBean(createReqVO, RelationsContractDO.class);
        contractInfo.setStatus(ContractInfoStatusEnum.LATEST.getStatus());
        contractMapper.insert(contractInfo);
        // 保存业务uuid到附件中
        this.saveFileList(createReqVO.getFileIdList(), infoId);
        // 返回
        return contractInfo.getId();
    }

    @Override
    @TenantIgnore
    public void updateContractInfo(RelationsContractSaveReqVO updateReqVO) {
        // 校验存在
        validateContractInfoExists(updateReqVO.getId());
        // 保存业务uuid到附件中
        this.saveFileList(updateReqVO.getFileIdList(), updateReqVO.getContractId());
        // 更新
        RelationsContractDO updateObj = BeanUtils.toBean(updateReqVO, RelationsContractDO.class);
        contractMapper.updateById(updateObj);
    }

    @Override
    @TenantIgnore
    public void deleteContractInfo(Long id) {
        // 校验存在
        validateContractInfoExists(id);
        // 删除
        contractMapper.deleteById(id);
    }

    @TenantIgnore
    private void validateContractInfoExists(Long id) {
        if (contractMapper.selectById(id) == null) {
            throw exception(RELATIONS_CONTRACT_INFO_NOT_EXISTS);
        }
    }

    @Override
    @TenantIgnore
    public RelationsContractDO getContractInfo(Long id) {
        return contractMapper.selectById(id);
    }

    @Override
    @TenantIgnore
    public RelationsContractRespVO getById(Long id) {
        // 第一步：检查ID是否为空
        if (id == null) {
            throw new IllegalArgumentException("ID cannot be null");
        }
        // 第二步：从数据库获取数据
        RelationsContractDO contractDO = contractMapper.selectById(id);
        if (contractDO == null) {
            throw exception(RELATIONS_CONTRACT_INFO_NOT_EXISTS);
        }
        // 第三步：将DO对象转换为VO对象
        RelationsContractRespVO respVO = BeanUtils.toBean(contractDO, RelationsContractRespVO.class);
        if (respVO == null) {
            // 理论上BeanUtils.toBean不应该返回null，但为了安全起见，还是检查一下
            throw new RuntimeException("Failed to convert DO to VO");
        }
        // 第四步：获取附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(contractDO.getContractId());
        if (fileList != null) {
            // 如果fileList不为null，则设置到VO对象中
            respVO.setFileList(fileList);
        }
        // 第五步：返回VO对象
        return respVO;
    }

    @Override
    @TenantIgnore
    public PageResult<RelationsContractDO> getContractInfoPage(RelationsContractPageReqVO pageReqVO) {
        return contractMapper.selectPage(pageReqVO);
    }

    @Override
    @TenantIgnore
    public int effectiveContractInfo() {
        // 1. 查询状态为已完成且已到合同开始日期的合同
        List<RelationsContractDO> doneContracts = contractMapper.selectListByStatusAndEffectiveDateGe(
                null, LocalDate.now());
        if (CollUtil.isEmpty(doneContracts)) {
            return 0;
        }
        // 2. 收集所有需要查询的员工ID
        Set<Long> employeeIds = new HashSet<>();
        for (RelationsContractDO contract : doneContracts) {
            employeeIds.add(contract.getEmployeeId());
        }
        // 3. 一次性查询所有员工ID对应的“有效”状态合同
        List<RelationsContractDO> contracts = contractMapper.selectListByStatusAndEmployeeIds(
                null, employeeIds);
        Map<Long, List<RelationsContractDO>> employeeToEffectiveContracts = contracts.stream()
                .collect(Collectors.groupingBy(RelationsContractDO::getEmployeeId));
        // 4. 更新状态
        List<RelationsContractDO> toUpdate = new ArrayList<>();
        for (RelationsContractDO contract : doneContracts) {
            List<RelationsContractDO> effectives = employeeToEffectiveContracts.getOrDefault(contract.getEmployeeId(), Collections.emptyList());
//            List<RelationsContractInfoDO> effectives = contractInfoMapper.selectListByStatusAndEmployeeId(ContractInfoStatusEnum.EFFECTIVE.getStatus(), contract.getEmployeeId());
            // 标记所有当前有效的合同为已弃用
            for (RelationsContractDO effective : effectives) {
                effective.setStatus(ContractInfoStatusEnum.HISTORY.getStatus());
                toUpdate.add(effective);
            }
            // 标记当前合同为有效
            contract.setStatus(ContractInfoStatusEnum.LATEST.getStatus());
            toUpdate.add(contract);
        }
        // 5. 批量更新
        if (!toUpdate.isEmpty()) {
            contractMapper.updateBatch(toUpdate);
        }
        // 返回更新的合同数量
        return doneContracts.size();
    }

    /**
     * 保存业务uuid到附件中
     */
    @TenantIgnore
    private void saveFileList(List<Long> fileIdList, String serviceId) {
        if (CollectionUtil.isNotEmpty(fileIdList) && StrUtil.isNotEmpty(serviceId)) {
            fileApi.updateFileBiz(fileIdList, serviceId);
        }
    }

}