package cn.iocoder.yudao.module.relations.service.conversion;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.tenant.core.aop.TenantIgnore;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import cn.iocoder.yudao.module.relations.controller.admin.conversion.vo.*;
import cn.iocoder.yudao.module.relations.dal.dataobject.conversion.RelationsConversionDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import cn.iocoder.yudao.module.relations.dal.mysql.conversion.RelationsConversionMapper;

import javax.annotation.Resource;

import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.relations.enums.ErrorCodeConstants.*;

/**
 * 转正信息 Service 实现类
 *
 * @author zhaopq
 */
@Service
@Validated
public class RelationsConversionServiceImpl implements RelationsConversionService {

    @Resource
    private RelationsConversionMapper conversionMapper;
    @Resource
    private FileApi fileApi;

    @Override
    @TenantIgnore
    public Long createConversion(RelationsConversionSaveReqVO createReqVO) {
        // 插入
        String infoId = IdUtil.fastSimpleUUID();
        createReqVO.setConversionId(infoId);
        RelationsConversionDO conversion = BeanUtils.toBean(createReqVO, RelationsConversionDO.class);
        conversionMapper.insert(conversion);
        // 保存业务uuid到附件中
        this.saveFileList(createReqVO.getFileIdList(), infoId);
        // 返回
        return conversion.getId();
    }

    @Override
    @TenantIgnore
    public void updateConversion(RelationsConversionSaveReqVO updateReqVO) {
        // 校验存在
        validateConversionExists(updateReqVO.getId());
        // 保存业务uuid到附件中
        this.saveFileList(updateReqVO.getFileIdList(), updateReqVO.getConversionId());
        // 更新
        RelationsConversionDO updateObj = BeanUtils.toBean(updateReqVO, RelationsConversionDO.class);
        conversionMapper.updateById(updateObj);
    }

    @Override
    @TenantIgnore
    public void deleteConversion(Long id) {
        // 校验存在
        validateConversionExists(id);
        // 删除
        conversionMapper.deleteById(id);
    }

    @TenantIgnore
    private void validateConversionExists(Long id) {
        if (conversionMapper.selectById(id) == null) {
            throw exception(RELATIONS_CONVERSION_NOT_EXISTS);
        }
    }

    @Override
    @TenantIgnore
    public RelationsConversionDO getConversion(Long id) {
        return conversionMapper.selectById(id);
    }

    @Override
    @TenantIgnore
    public RelationsConversionRespVO getById(Long id) {
        // 第一步：检查ID是否为空
        if (id == null) {
            throw new IllegalArgumentException("ID cannot be null");
        }
        // 第二步：从数据库获取数据
        RelationsConversionDO conversionDO = conversionMapper.selectById(id);
        if (conversionDO == null) {
            throw exception(RELATIONS_CONVERSION_NOT_EXISTS);
        }
        // 第三步：将DO对象转换为VO对象
        RelationsConversionRespVO respVO = BeanUtils.toBean(conversionDO, RelationsConversionRespVO.class);
        if (respVO == null) {
            // 理论上BeanUtils.toBean不应该返回null，但为了安全起见，还是检查一下
            throw new RuntimeException("Failed to convert DO to VO");
        }
        // 第四步：获取附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(conversionDO.getConversionId());
        if (fileList != null) {
            // 如果fileList不为null，则设置到VO对象中
            respVO.setFileList(fileList);
        }
        // 第五步：返回VO对象
        return respVO;
    }

    @Override
    @TenantIgnore
    public PageResult<RelationsConversionDO> getConversionPage(RelationsConversionPageReqVO pageReqVO) {
        return conversionMapper.selectPage(pageReqVO);
    }

    /**
     * 保存业务uuid到附件中
     */
    @TenantIgnore
    private void saveFileList(List<Long> fileIdList, String serviceId) {
        if (CollectionUtil.isNotEmpty(fileIdList) && StrUtil.isNotEmpty(serviceId)) {
            fileApi.updateFileBiz(fileIdList, serviceId);
        }
    }

}