package cn.iocoder.yudao.module.relations.service.entry;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.tenant.core.aop.TenantIgnore;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.module.relations.controller.admin.entry.vo.*;
import cn.iocoder.yudao.module.relations.dal.dataobject.entry.RelationsEntryDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import cn.iocoder.yudao.module.relations.dal.mysql.entry.RelationsEntryMapper;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.relations.enums.ErrorCodeConstants.*;

/**
 * 入职信息 Service 实现类
 *
 * @author zhaopq
 */
@Service
@Validated
public class RelationsEntryServiceImpl implements RelationsEntryService {

    @Resource
    private RelationsEntryMapper entryMapper;
    @Resource
    private FileApi fileApi;

    @Override
    @TenantIgnore
    public Long createEntry(RelationsEntrySaveReqVO createReqVO) {
        // 插入
        String infoId = IdUtil.fastSimpleUUID();
        createReqVO.setEntryId(infoId);
        RelationsEntryDO entry = BeanUtils.toBean(createReqVO, RelationsEntryDO.class);
        entryMapper.insert(entry);
        // 保存业务uuid到附件中
        this.saveFileList(createReqVO.getFileIdList(), infoId);
        // 返回
        return entry.getId();
    }

    @Override
    @TenantIgnore
    public void updateEntry(RelationsEntrySaveReqVO updateReqVO) {
        // 校验存在
        validateEntryExists(updateReqVO.getId());
        // 保存业务uuid到附件中
        this.saveFileList(updateReqVO.getFileIdList(), updateReqVO.getEntryId());
        // 更新
        RelationsEntryDO updateObj = BeanUtils.toBean(updateReqVO, RelationsEntryDO.class);
        entryMapper.updateById(updateObj);
    }

    @Override
    @TenantIgnore
    public void deleteEntry(Long id) {
        // 校验存在
        validateEntryExists(id);
        // 删除
        entryMapper.deleteById(id);
    }

    @TenantIgnore
    private void validateEntryExists(Long id) {
        if (entryMapper.selectById(id) == null) {
            throw exception(RELATIONS_ENTRY_NOT_EXISTS);
        }
    }

    @Override
    @TenantIgnore
    public RelationsEntryDO getEntry(Long id) {
        return entryMapper.selectById(id);
    }

    @Override
    @TenantIgnore
    public RelationsEntryRespVO getById(Long id) {
        // 第一步：检查ID是否为空
        if (id == null) {
            throw new IllegalArgumentException("ID cannot be null");
        }
        // 第二步：从数据库获取数据
        RelationsEntryDO entryDO = entryMapper.selectById(id);
        if (entryDO == null) {
            throw exception(RELATIONS_ENTRY_NOT_EXISTS);
        }
        // 第三步：将DO对象转换为VO对象
        RelationsEntryRespVO respVO = BeanUtils.toBean(entryDO, RelationsEntryRespVO.class);
        if (respVO == null) {
            // 理论上BeanUtils.toBean不应该返回null，但为了安全起见，还是检查一下
            throw new RuntimeException("Failed to convert DO to VO");
        }
        // 第四步：获取附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(entryDO.getEntryId());
        if (fileList != null) {
            // 如果fileList不为null，则设置到VO对象中
            respVO.setFileList(fileList);
        }
        // 第五步：返回VO对象
        return respVO;
    }

    @Override
    @TenantIgnore
    public PageResult<RelationsEntryDO> getEntryPage(RelationsEntryPageReqVO pageReqVO) {
        return entryMapper.selectPage(pageReqVO);
    }

    /**
     * 保存业务uuid到附件中
     */
    @TenantIgnore
    private void saveFileList(List<Long> fileIdList, String serviceId) {
        if (CollectionUtil.isNotEmpty(fileIdList) && StrUtil.isNotEmpty(serviceId)) {
            fileApi.updateFileBiz(fileIdList, serviceId);
        }
    }

}