package cn.iocoder.yudao.module.relations.service.turnover;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.tenant.core.aop.TenantIgnore;
import cn.iocoder.yudao.module.infra.api.file.FileApi;
import cn.iocoder.yudao.module.infra.api.file.dto.FileDTO;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import cn.iocoder.yudao.module.relations.controller.admin.turnover.vo.*;
import cn.iocoder.yudao.module.relations.dal.dataobject.turnover.RelationsTurnoverDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import cn.iocoder.yudao.module.relations.dal.mysql.turnover.RelationsTurnoverMapper;

import javax.annotation.Resource;

import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.relations.enums.ErrorCodeConstants.*;

/**
 * 离职信息 Service 实现类
 *
 * @author zhaopq
 */
@Service
@Validated
public class RelationsTurnoverServiceImpl implements RelationsTurnoverService {

    @Resource
    private RelationsTurnoverMapper turnoverMapper;
    @Resource
    private FileApi fileApi;

    @Override
    @TenantIgnore
    public Long createTurnover(RelationsTurnoverSaveReqVO createReqVO) {
        // 插入
        String infoId = IdUtil.fastSimpleUUID();
        createReqVO.setTurnoverId(infoId);
        RelationsTurnoverDO turnover = BeanUtils.toBean(createReqVO, RelationsTurnoverDO.class);
        turnoverMapper.insert(turnover);
        // 保存业务uuid到附件中
        this.saveFileList(createReqVO.getFileIdList(), infoId);
        // 返回
        return turnover.getId();
    }

    @Override
    @TenantIgnore
    public void updateTurnover(RelationsTurnoverSaveReqVO updateReqVO) {
        // 校验存在
        validateTurnoverExists(updateReqVO.getId());
        // 保存业务uuid到附件中
        this.saveFileList(updateReqVO.getFileIdList(), updateReqVO.getTurnoverId());
        // 更新
        RelationsTurnoverDO updateObj = BeanUtils.toBean(updateReqVO, RelationsTurnoverDO.class);
        turnoverMapper.updateById(updateObj);
    }

    @Override
    @TenantIgnore
    public void deleteTurnover(Long id) {
        // 校验存在
        validateTurnoverExists(id);
        // 删除
        turnoverMapper.deleteById(id);
    }

    @TenantIgnore
    private void validateTurnoverExists(Long id) {
        if (turnoverMapper.selectById(id) == null) {
            throw exception(RELATIONS_TURNOVER_NOT_EXISTS);
        }
    }

    @Override
    @TenantIgnore
    public RelationsTurnoverDO getTurnover(Long id) {
        return turnoverMapper.selectById(id);
    }

    @Override
    @TenantIgnore
    public RelationsTurnoverRespVO getById(Long id) {
        // 第一步：检查ID是否为空
        if (id == null) {
            throw new IllegalArgumentException("ID cannot be null");
        }
        // 第二步：从数据库获取数据
        RelationsTurnoverDO turnoverDO = turnoverMapper.selectById(id);
        if (turnoverDO == null) {
            throw exception(RELATIONS_TURNOVER_NOT_EXISTS);
        }
        // 第三步：将DO对象转换为VO对象
        RelationsTurnoverRespVO respVO = BeanUtils.toBean(turnoverDO, RelationsTurnoverRespVO.class);
        if (respVO == null) {
            // 理论上BeanUtils.toBean不应该返回null，但为了安全起见，还是检查一下
            throw new RuntimeException("Failed to convert DO to VO");
        }
        // 第四步：获取附件列表
        List<FileDTO> fileList = fileApi.getFileDTOListByBiz(turnoverDO.getTurnoverId());
        if (fileList != null) {
            // 如果fileList不为null，则设置到VO对象中
            respVO.setFileList(fileList);
        }
        // 第五步：返回VO对象
        return respVO;
    }

    @Override
    @TenantIgnore
    public PageResult<RelationsTurnoverDO> getTurnoverPage(RelationsTurnoverPageReqVO pageReqVO) {
        return turnoverMapper.selectPage(pageReqVO);
    }

    /**
     * 保存业务uuid到附件中
     */
    @TenantIgnore
    private void saveFileList(List<Long> fileIdList, String serviceId) {
        if (CollectionUtil.isNotEmpty(fileIdList) && StrUtil.isNotEmpty(serviceId)) {
            fileApi.updateFileBiz(fileIdList, serviceId);
        }
    }

}